#include <stdlib.h>
#include "anamod.h"
#include "anamodsalsamodules.h"
#include "petscmat.h"

/*
 * test the structure category
 */
int main(int argc,char **argv)
{
  Mat A; 
  PetscScalar v; PetscInt i,j;
  AnalysisItem result;
  PetscBool flg; PetscErrorCode ierr;
  PetscInitialize(&argc,&argv,0,0);
  ierr = AnaModInitialize(); CHKERRQ(ierr);

  ierr = RegisterStructureModules(); CHKERRQ(ierr);

  ierr = MatCreateSeqAIJ(MPI_COMM_SELF,3,3,3,0,&A); CHKERRQ(ierr);
  i = 0; j = 0; v = 1.;
  ierr = MatSetValues(A,1,&i,1,&j,&v,INSERT_VALUES); CHKERRQ(ierr);
  i = 1; j = 0; v = 1.;
  ierr = MatSetValues(A,1,&i,1,&j,&v,INSERT_VALUES); CHKERRQ(ierr);
  i = 1; j = 1; v = 1.;
  ierr = MatSetValues(A,1,&i,1,&j,&v,INSERT_VALUES); CHKERRQ(ierr);
  i = 2; j = 1; v = 1.;
  ierr = MatSetValues(A,1,&i,1,&j,&v,INSERT_VALUES); CHKERRQ(ierr);
  ierr = MatAssemblyBegin(A,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);
  ierr = MatAssemblyEnd(A,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);

  ierr = ComputeQuantity
    ((AnaModNumericalProblem)A,"structure","n-dummy-rows",
     &result,PETSC_NULL,&flg); CHKERRQ(ierr);
  if (!flg)
    SETERRQ(PETSC_COMM_SELF,1,"Could not compute #dummy rows");
  if (result.i!=2)
    SETERRQ1(PETSC_COMM_SELF,1,"found wrong number of dummy rows: %d",result.i);
  ierr = ComputeQuantity
    ((AnaModNumericalProblem)A,"structure","dummy-rows",
     &result,&i,&flg); CHKERRQ(ierr);
  if (!flg)
    SETERRQ(PETSC_COMM_SELF,1,"Could not compute dummy rows");
  if (i!=2)
    SETERRQ1(PETSC_COMM_SELF,1,"Should have got 2 as length of dummy row array, got %d",i);
  if (result.ii[0]!=0 || result.ii[1]!=2)
    SETERRQ2(PETSC_COMM_SELF,1,"Wrong dummy row numbers %d,%d",result.ii[0],result.ii[1]);

  ierr = MatDestroy(&A); CHKERRQ(ierr);
  ierr = AnaModFinalize(); CHKERRQ(ierr);
  PetscFinalize();
  PetscFunctionReturn(0);
}
