#include <stdlib.h>
#include "memory.h"
#include "anamod.h"

/*
 * Array modules
 */
int IntModule
(AnaModNumericalProblem problem,AnalysisItem *v,int *lv,PetscBool *flg)
{
  PetscErrorCode ierr;
  int len = *(int*)problem, *iar,i;
  PetscFunctionBegin;
  printf("Creating an int array of length %d\n",len);
  ierr = PetscMalloc(len*sizeof(int),&iar); CHKERRQ(ierr);
  for (i=0; i<len; i++) iar[i] = 2+i;
  v->ii = iar; if (lv) *lv = len; *flg = PETSC_TRUE;
  PetscFunctionReturn(0);
}

int RealModule
(AnaModNumericalProblem problem,AnalysisItem *v,int *lv,PetscBool *flg)
{
  PetscErrorCode ierr;
  int len = *(int*)problem,i; double *rar;
  PetscFunctionBegin;
  printf("Creating a real array of length %d\n",len);
  ierr = PetscMalloc(len*sizeof(double),&rar); CHKERRQ(ierr);
  for (i=0; i<len; i++) rar[i] = 2.1*(i+1);
  v->rr = rar; if (lv) *lv = len; *flg = PETSC_TRUE;
  PetscFunctionReturn(0);
}

int main(int argc,char **argv)
{
  AnalysisItem result; int l,lv;
  AnalysisDataType t;
  PetscBool flg; PetscErrorCode ierr;

  PetscInitialize(&argc,&argv,0,0);
  ierr = AnaModInitialize(); CHKERRQ(ierr);

  ierr = RegisterModule
    ("cat1","int1",ANALYSISINTARRAY,&IntModule); CHKERRQ(ierr);
  l = 5;
  ierr = ComputeQuantity
    ((AnaModNumericalProblem)&l,"cat1","int1",
     &result,&lv,&flg); CHKERRQ(ierr);
  if (!flg)
    printf("failed to compute int array\n");
  else
    printf("computed int %d\n",result.ii[0]);
  ierr = PetscFree(result.ii); CHKERRQ(ierr);

  ierr = RegisterModule
    ("cat1","int2",ANALYSISDBLARRAY,&RealModule); CHKERRQ(ierr);
  l = 3;
  ierr = ComputeQuantity
    ((AnaModNumericalProblem)&l,"cat1","int2",
     &result,&lv,&flg); CHKERRQ(ierr);
  if (!flg)
    printf("failed to compute real array\n");
  else
    printf("computed real %e\n",result.rr[0]);
  ierr = PetscFree(result.rr); CHKERRQ(ierr);

  ierr = GetDataType("cat1","int1",&t,NULL); CHKERRQ(ierr);
  if (t!=ANALYSISINTARRAY) SETERRQ(PETSC_COMM_SELF,1,"Was expecting int type");
  ierr = GetDataType("cat1","int2",&t,NULL); CHKERRQ(ierr);
  if (t!=ANALYSISDBLARRAY) SETERRQ(PETSC_COMM_SELF,1,"Was expecting double type");

  ierr = AnaModFinalize(); CHKERRQ(ierr);

  PetscFinalize();
  return 0;
}
